@@ -648,7 +648,7 @@ static void init_Module(server_rec *s, pool *p)
     ap_add_version_component(buf);
     cp = __SSLeay SSLeay_version(SSLEAY_VERSION);
     for (; !isdigit(*cp); cp++);
-    strcpy(buf, "SSLeay/");
+    ap_cpystrn(buf, "SSLeay/", sizeof(buf));
     for (cp2 = buf + strlen(buf); *cp != ' ' && *cp != NUL;)
         *cp2++ = *cp++;
     *cp2 = NUL;
@@ -692,7 +692,7 @@ static void init_GetCertAndKey(server_rec *s, pool *p, SSLConfigRec *pConfig)
      *  Check for problematic re-initializations
      */
     if (pConfig->px509Certificate) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, s,
+        ap_log_error(APLOG_MARK, APLOG_ERR|APLOG_NOERRNO, s,
                      "mod_ssl: Illegal attempt to re-initialise SSL for server %s",
                      s->server_hostname);
         ap_clear_pool(p);
@@ -744,9 +744,10 @@ static void init_GetCertAndKey(server_rec *s, pool *p, SSLConfigRec *pConfig)
      * Read server certificate
      */
     if ((f = ap_pfopen(p, pConfig->szCertificateFile, "r")) == NULL) {
-        sprintf(szPath, "%s/%s", __SSLeay X509_get_default_cert_dir(),
-                pConfig->szCertificateFile);
-        if ((f = fopen(szPath, "r")) == NULL) {
+        ap_snprintf(szPath, sizeof(szPath), "%s/%s", 
+                    __SSLeay X509_get_default_cert_dir(),
+                    pConfig->szCertificateFile);
+        if ((f = ap_pfopen(p, szPath, "r")) == NULL) {
             ap_log_error(APLOG_MARK, APLOG_ERR, s,
                          "mod_ssl: Can't open SSL server certificate file %s, nor %s",
                          pConfig->szCertificateFile, szPath);
@@ -756,7 +757,7 @@ static void init_GetCertAndKey(server_rec *s, pool *p, SSLConfigRec *pConfig)
     }
     else {
         /* in case it also contains the key */
-        strcpy(szPath, pConfig->szCertificateFile);     
+        ap_cpystrn(szPath, pConfig->szCertificateFile, sizeof(szPath));
     }
     pConfig->px509Certificate = __SSLeay X509_new();
     if (!__SSLeay PEM_read_X509(f, &pConfig->px509Certificate, NULL)) {
@@ -775,10 +776,11 @@ static void init_GetCertAndKey(server_rec *s, pool *p, SSLConfigRec *pConfig)
      */
     if (pConfig->szKeyFile)
         if (*pConfig->szKeyFile == '/')
-            strcpy(szPath, pConfig->szKeyFile);
+            ap_cpystrn(szPath, pConfig->szKeyFile, sizeof(szPath));
         else
-            sprintf(szPath, "%s/private/%s", 
-                    __SSLeay X509_get_default_cert_area(), pConfig->szKeyFile);
+            ap_snprintf(szPath, sizeof(szPath), "%s/private/%s", 
+                        __SSLeay X509_get_default_cert_area(), 
+                        pConfig->szKeyFile);
     /* Otherwise the path already contains the name of the certificate file */
     if ((f = ap_pfopen(p, szPath, "r")) == NULL) {
         ap_log_error(APLOG_MARK, APLOG_ERR, s,
@@ -1096,9 +1098,9 @@ static int ssl_hook_Fixup(request_rec *r)
             break;
         }
     }
-    sprintf(buf, "%d", keysize);
+    ap_snprintf(buf, sizeof(buf), "%d", keysize);
     ap_table_set(e, "HTTPS_KEYSIZE", buf);
-    sprintf(buf, "%d", secretkeysize);
+    ap_snprintf(buf, sizeof(buf), "%d", secretkeysize);
     ap_table_set(e, "HTTPS_SECRETKEYSIZE", buf);
 
     if (r->connection->client->szClientX509) {
@@ -1253,7 +1255,7 @@ static void ssl_int_ExpandCert(table *pEnv, char *szPrefix, char *szCert)
     ap_table_set(pEnv, var, szCert);
 
     /* FIXME - strtok() and strcspn() may cause problems on some systems - Q */
-    strcpy(buf, szCert);
+    ap_cpystrn(buf, szCert, sizeof(buf));
     for (s = strtok(buf, "/"); s != NULL; s = strtok(NULL, "/")) {
         int n = strcspn(s, "=");
         s[n] = NUL;
@@ -1333,7 +1335,7 @@ static int CacheLauncher(void *cmd, child_info *pInfo)
 
     ap_error_log2stderr(g_pServer);
     dup2(STDERR_FILENO, STDOUT_FILENO);
-    sprintf(a1, "%d", ap_user_id);
+    ap_snprintf(a1, sizeof(a1), "%d", ap_user_id);
     ap_cleanup_for_exec();
     execl(cmd, "ssl_gcache", a1, g_szCacheServerPort, NULL);
     perror(cmd);
